/* SPH80: ࠭  80 㣮묨 ࠭ﬨ  பᨬ樨 .
	    ,   ࠢ  -
	 ࠭   20 ࠢ஭  㣮묨  ࠭ﬨ.  
	    㣮쭨 ᯮ    -
	 㣮쭨,   20  4 = 80  㣮쭨 ( 㦥 
	 ࠢ஭).   ࠢ஭ 㣮쭨 -
	  । 窠     ஭ ஥  ,
	   ᠭ .  쪨 㣮쭨 -
	  ⥬ ᮥ ᥤ 祪 १ אַ .

	 Polyhedron with 80 triangular faces  to approximate  a sphere.
	 It is based on an icosahedron, which is  a regular  polyhedron
	 with  20 equilateral triangles  as bounding faces.  Instead of
	 each of  these triangles,  four  smaller  triangles  are  used,
	 which  gives  20 x 4 = 80  of  these  smaller  triangles  (not
	 equilateral). For each equilateral triangle of the icosahedron
	 the midpoints of its three sides are projected onto the sphere
	 in which the icosahedron fits.  We  obtain  the  four  smaller
	 triangles by connecting neighboring points.
*/
#include <stdio.h>
#include <math.h>
#include <process.h>

void point(int i, float x, float y, float z);
void subdivide(int A, int B, int C);
void storeface(int i, int j, int k);
void midpnt(int B, int C, int *pP, float x, float y, float z);

int npoints;
struct {float x, y, z;} pnt[43];

int nface=0;
struct {int i, j, k;} fc[80];

int nedge=0;
struct {int i, j, P;} edges[120];

FILE *fp;

main()
{ int i, l;
  double pi, r, alpha, tau;
  pi = 4 * atan(1.0);
  tau = (sqrt(5.0) + 1)/2;  /* tau = 2 * cos(pi/5)  */
  r = tau - 0.5;
  printf /* "A sphere (with radius 1) is approximated " */
	("\n ( ࠤᮬ 1) பᨬ ꥪ⮬,\n");
  printf /* "by an object with 80 triangular bounding" */
	("࠭祭 80 㣮묨 ࠭ﬨ. \n ");
  printf /* "faces. The results are written to the " */
	("뢠  室 䠩 SPH80.DAT,\n ");
  printf /* "object file SPH80.DAT (which can be read by D3D)." */
	("⥬   ⠭ ணࠬ D3D.\n");
  fp = fopen("SPH80.DAT", "w");
  point(1, 0.0, 0.0, 1.0); /* North pole */   /*   */
  for (i=0; i<5; i++)
  { alpha = -pi/5 + i * pi/2.5;
		  /*   ࠤ (In degrees): -36 + i * 72 */
    point(2+i, cos(alpha)/r, sin(alpha)/r, 0.5/r);
  }
  for (i=0; i<5; i++)
  { alpha = i * pi/2.5;
    point(7+i, cos(alpha)/r, sin(alpha)/r, -0.5/r);
  }
  point(12, 0.0, 0.0, -1.0); /* South pole */   /*   */
  npoints = 12;
  for (i=0; i<5; i++) subdivide(1, 2+i, i<4 ? 3+i : 2);
  for (i=0; i<5; i++) subdivide(2+i, 7+i, i<4 ? 3+i : 2);
  for (i=0; i<5; i++)
            subdivide(7+i, i<4 ? 8+i : 7, i<4 ? i+3 : 2);
  for (i=0; i<5; i++) subdivide(i+7, 12, i<4 ? i+8 : 7);
  fprintf(fp, "Faces = ࠭:\n");
  for (l=0; l<80; l++)
    fprintf(fp, "%d %d %d.\n", fc[l].i, fc[l].j, fc[l].k);
  fclose(fp);
}

void midpnt(int B, int C, int *pP, float x, float y, float z)

/*    窠 (x, y, z)  । 窮 १ BC.
    ᫨   設,     뢠 
    ꥪ 䠩   ᯮ짮   設.
    ᫨ ,  室   設.  設 
     ਯᠭ *pP   砥.

      Point (x, y, z) is midpoint of BC.
    If it is a new vertex, store it and write it to the object file,
    using a new vertex number. If not, find its vertex number.
    The vertex number is to be assigned to *pP anyway.
*/
{ int tmp, e;
  if (C < B) {tmp = B; B = C; C = tmp;}
  /* B, C in increasing order, for the sake of uniqueness */
  /* B  C  浪 ⠭ ࠤ 㭨䨪樨	  */
  for (e=0; e<nedge; e++)
    if (edges[e].i == B && edges[e].j == C) break;
  if (e == nedge)   /* Not found, so we have a new vertex */
	     /*  㦥,    設 */
  { edges[e].i = B; edges[e].j = C;
    edges[e].P = *pP = ++npoints;
    nedge++;
    point(*pP, x, y, z);
  } else *pP = edges[e].P;
     /* Edge BC has been dealt with before, so the vertex is not new
	 BC 㦥 ᬠਢ ࠭, ᫥⥫쭮 設  
     */
}

void point(int i, float x, float y, float z)

{ fprintf(fp, "%d %f %f %f\n", i, x, y, z);
  pnt[i].x = x; pnt[i].y = y; pnt[i].z = z;
}

void subdivide(int A, int B, int C)

/*   㣮쭨 ABC    㣮쭨  */
/*  Divide triangle ABC into four smaller triangles        */

{ float xP, yP, zP, xQ, yQ, zQ, xR, yR, zR;
  int P, Q, R;
  static float d = -1;
    /* 祭 ६ d ࠢ -1 ⮫쪮  ࢮ 饭  ⮩
       㭪樨; ᫥ ࢮ 맮 ६ d 㤥 ᢮ 筮
       祭  ( 0  1),  ࠢ  ﭨ    ।
       窮  業஬ .  । 窨  ஥   
       ( ࠤᮬ 1) ⥬   d 祭  न.

	d is equal to -1 only before the first  call  of this  function;
       after this,  d will have  its correct  value  (between 0 and 1),
       namely  the distance between any midpoint and the center of the
       sphere. We project all midpoints onto the sphere (with radius 1)
       by dividing their coordinates by d.
    */
  xP = (pnt[B].x + pnt[C].x)/2;
  yP = (pnt[B].y + pnt[C].y)/2;
  zP = (pnt[B].z + pnt[C].z)/2;

  xQ = (pnt[C].x + pnt[A].x)/2;
  yQ = (pnt[C].y + pnt[A].y)/2;
  zQ = (pnt[C].z + pnt[A].z)/2;

  xR = (pnt[A].x + pnt[B].x)/2;
  yR = (pnt[A].y + pnt[B].y)/2;
  zR = (pnt[A].z + pnt[B].z)/2;

  if (d < 0) d = sqrt(xP*xP + yP*yP + zP*zP);
                                   /* 0 < d < 1 */
  xP /= d; yP /= d; zP /= d;
  xQ /= d; yQ /= d; zQ /= d;
  xR /= d; yR /= d; zR /= d;

  midpnt(B, C, &P, xP, yP, zP);
  midpnt(C, A, &Q, xQ, yQ, zQ);
  midpnt(A, B, &R, xR, yR, zR);

  storeface(A, R, Q);
  storeface(R, B, P);
  storeface(Q, P, C);
  storeface(Q, R, P);
}

void storeface(int i, int j, int k)

{ fc[nface].i = i;
  fc[nface].j = j;
  fc[nface].k = k;
  nface++;
}
